#!/usr/bin/perl
# 
# Generate NICER background PI spectrum based on "3C50_RGv5" library
#
my $taskname = "nibackgen3C50.pl";
my $taskvers = "0.3";
# v0.0 - initial version
# v0.1 - filters out HBG outliers
# v0.2 - enable list of FPMs to exclude
# v0.3 - incorporates some HEASoft 6.25-to-6.26.1 changes of other NICER  tasks
use HEACORE::HEAINIT;
# v0.4 - bug fix for many-GTI case, reduce number of "tmp" files created
# v0.5 - bug fix in mathpha_str subroutine

# ===================================
# Execute main subroutine, with error trapping
$status = 0;
my @tmpfiles = ();
my ($tmpfile, $atstatus, $cleanup);
$cleanup = 1;
eval {
    $status = headas_main(\&nibackgen3C50);
};
$atstatus = $@;

# ===================================
# Check for errors and report them to the console
if ($atstatus) {
    if ($status == 0) { $status = -1; }
    warn $atstatus;
}
exit $status;
exit 0;
# <========= Main task ends here

# ===================================
# Main subroutine

sub nibackgen3C50 {
    # Makes all environment variables available
    use Env;
    use Cwd;

    # The HEAUTILS module provides access to HDpar_stamp()
    # set_toolname(), set_toolversion(), and headas_clobberfile()
    use HEACORE::HEAUTILS;
    use HEACORE::PIL;
    # include the file specification functions
    use Astro::FITS::CFITSIO qw( :shortnames :constants );
    use POSIX qw(fmod);

    # Use the standard HEAdas methods for registering the toolname and version number to be
    # used in error reporting and in the record of parameter values written by HDpar_stamp
    set_toolname($taskname);
    set_toolversion($taskvers);

    # Prevent subsidiary tasks from pausing for input 
    $ENV{"HEADASNOQUERY"} = 1;
    $ENV{"HEADASPROMPT"} = "/dev/null";

    eval {
	$status = &nibackgen3C50_work();
    };

    # Special effort to remove scratch files
    if ($cleanup) {
	foreach $scratchfile (@tmpfiles) {
	    unlink "$scratchfile" if (-f $scratchfile);

	}
    }

    if ($@) {
	if ($status == 0) { $status = -1; }
	warn $@;
	return $status;
    }

    return $status;
}

sub nibackgen3C50_work {
    # Makes all environment variables available
    use Env;
    use Cwd;

    # The HEAUTILS module provides access to HDpar_stamp()
    # set_toolname(), set_toolversion(), and headas_clobberfile()
    use HEACORE::HEAUTILS;
    use HEACORE::PIL;
    # include the file specification functions
    use Astro::FITS::CFITSIO qw( :shortnames :constants );
    # User defined module which contains the Perl-CFITSIO interface functions 
    use SimpleFITS;
    use POSIX qw(fmod);

    # --------------- parameters
    my $chatter;
    $status = PILGetInt("chatter",$chatter);
    my $verbose = ($chatter >= 5)?(1):(0);
    # Ordered parameters, usually ask parameters which must come first
    my @parmlist = ("rootdir");
    my %parms = (
		 'rootdir' => \&PILGetString,
		 'obsid' => \&PILGetString,
		 'bkgdir' => \&PILGetString,
		 'calevtdir' => \&PILGetString,
		 'totspec' => \&PILGetString,
		 'bkgspec' => \&PILGetString,
		 'ufafile' => \&PILGetString,
		 'clfile' => \&PILGetString,
		 'mkfile' => \&PILGetString,
		 'fpmofflist' => \&PILGetString,
		 'dtmin' => \&PILGetReal,
		 'dtmax' => \&PILGetReal,
		 'hbgcut' => \&PILGetReal,
		 'cleanup' => \&PILGetBool,
	         'clobber' => \&PILGetBool,
		 'history' => \&PILGetBool,
		);
    # Find parameter values (listed above)
    my ($parm, $func, $val);
    # ... first read ordered parameters, then anything else
    foreach $parm ( @parmlist, keys(%parms) ) {
      my $func = $parms{$parm};
      next if (ref($func) ne "CODE");  # Skip if we already did this parm
      undef($val);
      $status = &$func("$parm", $val);
      die "ERROR: could not retrieve parameter '$parm'" if ($status);
      $parms{$parm} = $val;
      print "$parm=$val\n" if ($verbose);
    }

    my $clobstr = ($parms{clobber}) ? "YES" : "NO";
    my $histstr = ($parms{history}) ? "YES" : "NO";
    # Global variable
    $cleanup = $parms{cleanup};
    my $cleanstr = ($cleanup) ? "YES" : "NO";

    print "$taskname $taskvers\n" if ($chatter >= 1);
    print "--------------------------------------------------------\n" if ($chatter >= 1);
    foreach my $k (sort keys(%parms)) {
      print "      $k = ".$parms{$k}."\n" if ($chatter >= 2);
    }

    my $rootdir = $parms{rootdir};
    my $obsid = $parms{obsid};
    #print "  ROOTDIR = ".$parms{rootdir}."\n" if ($chatter >= 2);

    if ( ($parms{rootdir} !~ m/^NONE$/i) && ($params{obsid} !~ m/^NONE$/i) ) {
        #print "  ???"."\n" if ($chatter >= 2);
        $indir = $rootdir . "/" . $obsid;
	#print "  INDIR = ".$indir."\n" if ($chatter >= 2);
	die "ERROR: input directory $indir does not exist" if (! -d $indir);
    }

    #my $indir = $rootdir . "/" . $obsid;
    #print "  INDIR = ".$indir."\n" if ($chatter >= 2);

    my $calevtdir = $parms{calevtdir};
    if ($parms{calevtdir} !~ m/^NONE$/i) {
	#print "$calevtdir \n";
	$calevtdir =~ s/\$OBSID/$obsid/g; $calevtdir =~ s/\$INDIR/$indir/g;
	#print "$calevtdir \n";
	die "ERROR: cl directory $calevtdir does not exist" if (! -d $calevtdir);
    }
    #print "$calevtdir \n";

    # Find the input event files
    my $ufafile = $parms{ufafile};
    my $clfile = $parms{clfile};
    $ufafile =~ s/\$CALEVTDIR/$calevtdir/g; $ufafile =~ s/\$OBSID/$obsid/g; $ufafile =~ s/\$INDIR/$indir/g;
    $ufafile .= ".gz" if ( ! -f $ufafile );
    die "ERROR: uf file $ufafile does not exist" if (! -f $ufafile);
    $clfile  =~ s/\$CALEVTDIR/$calevtdir/g; $clfile  =~ s/\$OBSID/$obsid/g; $clfile  =~ s/\$INDIR/$indir/g;
    $clfile .= ".gz" if ( ! -f $clfile );
    die "ERROR: cl file $clfile does not exist" if (! -f $clfile);

    my $mkfile = $parms{mkfile};
    if ($parms{mkfile} !~ m/^NONE$/i) {
	# Find the MKF file
	$mkfile =~ s/\$CALEVTDIR/$calevtdir/g; $mkfile =~ s/\$OBSID/$obsid/g; $mkfile =~ s/\$INDIR/$indir/g;
	$mkfile .= ".gz" if ( ! -f $mkfile );
	die "ERROR: could not find MKF file $mkfile (with or without .gz suffix)" if ( ! -f $mkfile );
    }

    my $totspec = $parms{totspec};
    my $bkgspec = $parms{bkgspec};

    my $fpms_off = ();
    if ($parms{fpmofflist} !~ m/^none$/i) {
	my @fpmlist = parse_fpm_list($parms{fpmofflist});
	print "  FPM List: $parms{fpmofflist}\n" if ($chatter >= 2);
	print "     --> ".join(',',@fpmlist)."\n" if ($chatter >= 4);
	for my $fpm (@fpmlist) {
	    push @fpms_off, $fpm;
	}
    }

    #for my $fpm (@fpms_off) {
	#print "  FPM: $fpm\n";
    #}

    # Background library -- applies only to "3C50_RGv5" version
    # IBG boundaries for background library
    my $nibg = 6;
    my @ibglo = (0.0,0.1,0.2,0.4,1.0,3.0);
    my @ibghi = (0.1,0.2,0.4,1.0,3.0,10.0);
    my $ibgmax = $ibghi[nibg-1];
    my $hrejmax = 5.0;
    my $nnz = 11;
    my @nzlo = (200.0,230.0,280.0,350.0,450.0,560.0,680.0,800.0,1000.0,1200.0,1500.0);
    my @nzhi = (230.0,280.0,350.0,450.0,560.0,680.0,800.0,1000.0,1200.0,1500.0,2000.0);
    my $nzmax = $nzhi[nnz-1];

    my $bkgdir = $parms{bkgdir};
    die "ERROR: bkg directory $bkgdir does not exist" if (! -d $bkgdir);

    # Change the delimiter in the original file from tab to comma 
    my $bg_group_ngt_table = "bg_group_3C50_RGngt_v5.table_notabs";
    $bg_group_ngt_table_file = $bkgdir . "/" . $bg_group_ngt_table;
    die "ERROR: bg_group_ngt_table file $bg_group_ngt_table_file does not exist" if (! -f $bg_group_ngt_table_file);

    # Change the delimiter in the original file from tab to comma 
    my $bg_group_day_table = "bg_group_3C50_RGday_v5.table_notabs";
    $bg_group_day_table_file = $bkgdir . "/" . $bg_group_day_table;
    die "ERROR: bg_group_day_table file $bg_group_day_table_file does not exist" if (! -f $bg_group_day_table_file);

    # Read the night-bkg table, extracting the library filenames, ids, and average ibg values
    my @bg_ngt_names= ();
    my @bg_ngt_ibgs= ();
    my @bg_ngt_ids= ();
    open (my $fh, "<", $bg_group_ngt_table_file) or die $!;
    while (my $row = <$fh>) {
	chomp $row;
	#print "$row\n";
        @fields = split(',', $row);
        my ($bg_group_ngt_table_name, $bg_group_ngt_table_ibg) = @fields[0,1];
        #my $bg_group_ngt_table_id = substr $bg_group_ngt_table_name, 23, 2;
        my $vmatch = 'v5_';
        my $idindex = 3 + index($bg_group_ngt_table_name, $vmatch);
        my $bg_group_ngt_table_id = substr $bg_group_ngt_table_name, $idindex, 2;
        #print "$bg_group_ngt_table_name $bg_group_ngt_table_ibg $bg_group_ngt_table_id\n";
	push @bg_ngt_names, $bg_group_ngt_table_name;
	push @bg_ngt_ibgs, $bg_group_ngt_table_ibg;
	push @bg_ngt_ids, $bg_group_ngt_table_id;
    }

    # Read the exposure times from the SPECTRUM extension of nighttime library files   
    # Get the HBG rates from the SPECTRUM extension of nighttime library files
    my $nbg_ngt_ids = scalar @bg_ngt_ids;
    my @bg_ngt_expos = (0) x ($nbg_ngt_ids);
    my @hbg_tcounts = (0) x ($nbg_ngt_ids);
    for (my $ii=0; $ii<$nbg_ngt_ids; $ii+=1) {
	$bg_ngt_spec = SimpleFITS->open("$bkgdir" . "/" . "$bg_ngt_names[$ii]", type=>"table", access=>"read");
	$bg_ngt_spec->move("SPECTRUM");
	$bg_ngt_spec->readkey("EXPOSURE",$bg_ngt_expo);
	$bg_ngt_expos[$ii] = $bg_ngt_expo;
        $bg_ngt_spec->readcol("COUNTS",{rows=>[1300,1500]},$hbg_counts_ref);
        @hbg_counts = @$hbg_counts_ref;
        foreach $hbgc (@hbg_counts) { $hbg_tcounts[$ii] += $hbgc };
	$hbg_tcounts[$ii] /= $bg_ngt_expo;
    }

    #for (my $ii=0; $ii<$nbg_ngt_ids; $ii+=1) {
	#print "$bg_ngt_ids[$ii], $bg_ngt_ibgs[$ii], $bg_ngt_names[$ii], $bg_ngt_expos[$ii]\n";
    #}

    # Read the day-bkg table, extracting the library filenames, ids, and average ibg values
    my @bg_day_names= ();
    my @bg_day_nzs= ();
    my @bg_day_ids= ();
    open (my $fh, "<", $bg_group_day_table_file) or die $!;
    while (my $row = <$fh>) {
	chomp $row;
	#print "$row\n";
        @fields = split(',', $row);
        my ($bg_group_day_table_name, $bg_group_day_table_nz) = @fields[0,1];
        #my $bg_group_day_table_id = substr $bg_group_day_table_name, 23, 2;
        my $vmatch = 'v5_';
        my $idindex = 3 + index($bg_group_day_table_name, $vmatch);
        my $bg_group_day_table_id = substr $bg_group_day_table_name, $idindex, 2;
        #print "$bg_group_day_table_name $bg_group_day_table_nz $bg_group_day_table_id\n";
	push @bg_day_names, $bg_group_day_table_name;
	push @bg_day_nzs, $bg_group_day_table_nz;
	push @bg_day_ids, $bg_group_day_table_id;
    }

    # Read the exposure times from the SPECTRUM extension of daytime library files 
    my $nbg_day_ids = scalar @bg_day_ids;
    my @bg_day_expos = (0) x ($nbg_day_ids);
    for (my $ii=0; $ii<$nbg_day_ids; $ii+=1) {
	$bg_day_spec = SimpleFITS->open("$bkgdir" . "/" . "$bg_day_names[$ii]", type=>"table", access=>"read");
	$bg_day_spec->move("SPECTRUM");
	$bg_day_spec->readkey("EXPOSURE",$bg_day_expo);
	$bg_day_expos[$ii] = $bg_day_expo;
    }

    #for (my $ii=0; $ii<$nbg_day_ids; $ii+=1) {
	#print "$bg_day_ids[$ii], $bg_day_nzs[$ii], $bg_day_names[$ii], $bg_day_expos[$ii]\n";
    #}

    # Filter bad detectors
    if ($parms{fpmofflist} =~ m/^none$/i) {
	$cmd = "ftcopy infile='$ufafile' ".
	    "outfile='tmp_ufafile_nobaddet.evt' ".
	    "copyall='yes' ".
	    "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    } else {
	my $detcnt = 0;
	my $detexpr = "";
	my $first = 1;
	foreach $fpm (@fpms_off) {
	    #print "  FPM: $fpm\n";
	    if ($first==1) {
		$expr_to_add = "(DET_ID!=".$fpm.")";
		$first = 0;
		#print "  E1: $expr_to_add\n";
	    } else {
		$expr_to_add = ".and.(DET_ID!=".$fpm.")";
		#print "  E2: $expr_to_add\n";
	    }
	    $detexpr = $detexpr.$expr_to_add 
	}		
	#print "  EXPR: $detexpr\n";
	$cmd = "ftcopy infile='$ufafile"."[EVENTS][".$detexpr."]' ".
	    "outfile='tmp_ufafile_nobaddet.evt' ".
	    "copyall='yes' ".
	    "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    }
    ## Filter bad detectors 14 and 34 
    #$cmd = "ftcopy infile='$ufafile"."[EVENTS][(DET_ID!=14).and.(DET_ID!=34)]' ".
      #"outfile='tmp_ufafile_nobaddet.evt' ".
      #"copyall='yes' ".
      #"clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcopy failed to filter bad detectors" if ($?);
    die "ERROR: file with bad detectors filtered not produced" if (! -f "tmp_ufafile_nobaddet.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_nobaddet.evt" };
    # Scaling factor, since library is scaled to 52 detectors
    my $nbad = scalar @fpms_off;
    my $livedet = 52;
    my $baddetrat = ($livedet-$nbad)/$livedet;
    #print "$livedet $nbad $baddetrat\n";

    # Extract slow chain ufa file
    $cmd = "niextract-events ".
      "filename='tmp_ufafile_nobaddet.evt[PI=-10:1800,EVENT_FLAGS=bxxx1x000]' ".
      "eventsout='tmp_ufafile_slowchain.evt' ".
      "regionfile='NONE' timefile='NONE' xcolf='DETX' ycolf='DETY' tcol='TIME' ".
      "events='EVENTS' gti='GTI' gtimerge='OR' "."colexpr='NONE' ".
      "minselectmb='2000' "."copyall='no' ".
      "cleanup='$cleanstr' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: task niextract-events failed to create slow chain ufa file" if ($?);
    die "ERROR: slow chain ufa file not produced" if (! -f "tmp_ufafile_slowchain.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_slowchain.evt" };

    # Filter slow chain ufa file to create trumpet-selected ufa file
    $cmd = "ftcopy ".
      "infile='tmp_ufafile_slowchain.evt[EVENTS][((ISNULL(PI_RATIO))||(PI_RATIO<(1.1+120/PI)))]' ".
      "outfile='tmp_ufafile_tsel.evt' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcopy failed to create trumpet-selected ufa file" if ($?);
    die "ERROR: trumpet-selected ufa file not produced" if (! -f "tmp_ufafile_tsel.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_tsel.evt" };

    # Filter trumpet-selected ufa file over IBG energies
    $cmd = "ftcopy ".
      "infile='tmp_ufafile_tsel.evt[EVENTS][PI=1500:1700]' ".
      "outfile='tmp_ufafile_tsel_ibg.evt' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcopy failed to create trumpet-selected IBG ufa file" if ($?);
    die "ERROR: trumpet-selected IBG ufa file not produced" if (! -f "tmp_ufafile_tsel_ibg.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_tsel_ibg.evt" };

    # Filter trumpet-selected ufa file over HBG energies
    $cmd = "ftcopy ".
      "infile='tmp_ufafile_tsel.evt[EVENTS][PI=1300:1500]' ".
      "outfile='tmp_ufafile_tsel_hbg.evt' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcopy failed to create trumpet-selected HBG ufa file" if ($?);
    die "ERROR: trumpet-selected HBG ufa file not produced" if (! -f "tmp_ufafile_tsel_hbg.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_tsel_hbg.evt" };

    # Filter slow chain ufa file to create hatchet-rejected ufa file
    $cmd = "ftcopy ".
      "infile='tmp_ufafile_slowchain.evt[EVENTS][((PI_RATIO>1.54)&&(PI=300:1800))] ' ".
      "outfile='tmp_ufafile_hrej.evt' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcopy failed to create hatchet-rejected ufa file" if ($?);
    die "ERROR: hatchet-rejecte ufa file not produced" if (! -f "tmp_ufafile_hrej.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_hrej.evt" };

   # Filter trumpet-selected ufa file over NZ energies to get slow-chain noise file
    $cmd = "ftcopy ".
      "infile='tmp_ufafile_tsel.evt[EVENTS][PI=0:20]' ".
      "outfile='tmp_ufafile_tsel_nz.evt' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcopy failed to create slow chain noise ufa file" if ($?);
    die "ERROR: slow chain noise ufa file not produced" if (! -f "tmp_ufafile_tsel_nz.evt");
    if ($cleanup) { push @tmpfiles, "tmp_ufafile_tsel_nz.evt" };

    # Redefine GTI attached to clfile so that for all final dt, dtmin<dt<=dtmax
    # Can add more options later (i.e., derive from mkf for some expression, any
    # input GTI)
    $makegtierror = createStartStopGTI($clfile,$parms{dtmin},$parms{dtmax},$histstr,$chatter);

    # Apply GTI to ufa file (without bad detectors), keeping all RMF-defined PI
    $cmd = "nicerclean ".
      "infile='tmp_ufafile_nobaddet.evt' ".
      "outfile='tmp_clfile_nobaddet_newgti.evt' ".
      "gtifile='tmp_newgti.gti' ".
      "pirange='0:1500' ".
      "trumpetfilt='yes' ".
      "filtexpr='EVENT_FLAGS=bxxx1x000' ".
      "cleanup='$cleanstr' "."history='$histstr' "."chatter='$chatter' "."clobber='yes' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: nicerclean failed to create new cleaned event file" if ($?);    
    die "ERROR: new cleaned event file not produced" if (! -f "tmp_clfile_nobaddet_newgti.evt");
    if ($cleanup) { push @tmpfiles, "tmp_clfile_nobaddet_newgti.evt" };

    $subgtierror = SubstituteGTI("tmp_clfile_nobaddet_newgti.evt","GTI","tmp_newgti.gti",$histstr,$chatter);

    # Get the number of GTI and delta-times in the event file with redifined GTI
    $newclean = SimpleFITS->open("tmp_clfile_nobaddet_newgti.evt", type=>"table", access=>"read");
    $newclean->move("EVENTS");
    $newclean->readkey("NAXIS2",$numnewclean);
    $newclean->readkey("EXPOSURE",$expnewclean);
    die "ERROR: no events in new cleaned event file" if ( $numnewclean<=0  );
    die "ERROR: zero good exposure in new cleaned event file" if ( $expnewclean<=0  );
    $newclean->move("GTI");
    $newclean->readkey("NAXIS2",$ngti);
    @indtgti = $newclean->calculate("STOP-START",{type=>TDOUBLE});
    $newclean->close();

    #for (my $ii=0; $ii<$ngti; $ii+=1) {
	#print "$indtgti[$ii]\n";
    #}

    # initialize weight arrays
    my @weights = (0) x ($nbg_ngt_ids);
    my @exp_weights = (0) x ($nbg_ngt_ids);
    my @supweights = (0) x ($nbg_day_ids);
    my $exp_bkg = 0.0;

    # temp text file for storing list of GTI with rates in the library range
    my $tmpgti = "tmp_gti.gti";
    # start with the full GTI and delete rows as appropriate
    $cmd = "ftcopy infile='tmp_newgti.gti' ".
	"outfile='$tmpgti' ".
	"copyall='yes' ".
	"clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: failed to copy redefined GTI file" if ($?);
    my $tmpgtilst = "tmp_gtilist.txt";
    open GTILIST, ">", $tmpgtilst;
    #print "$ngti\n" if ($chatter >= 3);
    # some bookeeping:
    my $nkeep = 0;
    my $nreject = 0;
    my $tkeep = 0.0;
    my $treject = 0.0;
    my $sumibg = 0.0;
    my $sumhrej = 0.0;
    my $sumhbg = 0.0;
    # loop over rows in gti file
    for (my $ii=1; $ii<$ngti+1; $ii+=1) { 
	my $id1 = 0;
	my $id2 = 0;
	my $id3 = 0;
	my $keep_flag = 0;
        # extract this row in the gti 
        $cmd = "ftcopy infile='tmp_clfile_nobaddet_newgti.evt[GTI][\#row ==$ii]' ".
          "outfile='tmp_clfile_nobaddet_newgti_$ii.gti' ".
          "copyall='no' ".
          "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
	print "$cmd\n" if ($chatter >= 3);
	system($cmd);
	die "ERROR: failed to extract GTI row $ii" if ($?);
	die "ERROR: ftcopy failed to produce GTI row $ii" if (! -f "tmp_clfile_nobaddet_newgti_$ii.gti");
        if ($cleanup) { push @tmpfiles, "tmp_clfile_nobaddet_newgti_"."$ii".".gti" };
        # Best match and scaling factor for bkg spectrum - night
        $ibgrate = RateGTI("tmp_ufafile_tsel_ibg.evt","tmp_clfile_nobaddet_newgti_$ii.gti",$indtgti[$ii-1],$baddetrat,$histstr,$chatter); 
        $hrejrate = RateGTI("tmp_ufafile_hrej.evt","tmp_clfile_nobaddet_newgti_$ii.gti",$indtgti[$ii-1],$baddetrat,$histstr,$chatter); 
        $hbgrate = RateGTI("tmp_ufafile_tsel_hbg.evt","tmp_clfile_nobaddet_newgti_$ii.gti",$indtgti[$ii-1],$baddetrat,$histstr,$chatter); 
        if ($ibgrate<=$ibgmax && $hrejrate<=$hrejmax) {
	    #print "$ii $ibgrate $ibgmax $hrejrate $hrejmax\n";
            for (my $jj=0; $jj<$nibg; $jj+=1) {
		my $flag = 0;
		if ($ibgrate>$ibglo[$jj] && $ibgrate<=$ibghi[$jj] && $flag==0) {
		    $id1 = $jj + 1;
                    $id2 = FindIBGID($id1,$hrejrate);
                    $flag = 1;
		    #print "$jj $ibgrate $ibglo[$jj] $ibghi[$jj] $hrejrate $id1 $id2 $flag\n";abs($hbgrate_uncorr-$hbg_bkg)
		}
	    }
	    $id12 = $id1.$id2;
	    #print "$ii $ibgrate $ibglo[$jj] $ibghi[$jj] $hrejrate $id1 $id2 $id12\n";
	    if ($id12 != 0) {
		#print "$id12\n";
		my $bgngtindx = -1;
                $bgngtindx++ until @bg_ngt_ids[$bgngtindx]==$id12 or $bgngtindx>$nbg_ngt_ids;
		$ibgbkg = $bg_ngt_ibgs[$bgngtindx];
		#print "$bgngtindx $ibgbkg\n";
                #$keep_flag = 1;
                # the rate corrected for number of MPU is used for matching, 
                # but the uncorrected rate is used for scaling
		$ibgrate_uncorr = $baddetrat * $ibgrate;
		$hbgrate_uncorr = $baddetrat * $hbgrate;
                $hbg_bkg = ($ibgrate_uncorr / $ibgbkg) * $hbg_tcounts[$bgngtindx];
		$hbg_tot = abs($hbgrate_uncorr-$hbg_bkg);
                #print "XXX $ibgbkg $ibgrate_uncorr $hbgrate_uncorr $hbg_bkg $hbg_tot \n";
                # proceed only if residual HBG is not too high
                if ($hbg_tot < $parms{hbgcut}) {
		    $keep_flag = 1;
		    $dweight = $indtgti[$ii-1] * $ibgrate_uncorr / $ibgbkg;
		    #$dweight = $indtgti[$ii-1];
		    $weights[$bgngtindx] += $dweight;
		    $exp_weights[$bgngtindx] += $indtgti[$ii-1];
		    #print "$bgngtindx $ibgbkg $id12 $ibgrate_uncorr $dweight $weights[$bgngtindx] $exp_weights[$bgngtindx]\n";
		    print GTILIST "tmp_clfile_nobaddet_newgti_$ii.gti\n";
		    # Best match bkg spectrum and scaling factor - day correction
		    $nzrate = RateGTI("tmp_ufafile_tsel_nz.evt","tmp_clfile_nobaddet_newgti_$ii.gti",$indtgti[$ii-1],$baddetrat,$histstr,$chatter);
                    #print "NZ $nzrate \n";
		    for (my $kk=0; $kk<$nnz; $kk+=1) {
			my $sflag = 0;
			# Use last library entry for arbitrarily high nz
			if ($nzrate > $nzmax) {
			    $id33 = "1"."2"; 
			} elsif ($nzrate>$nzlo[$kk] && $nzrate<=$nzhi[$kk] && $sflag==0) {
			    $id3 = $kk+1;
			    $sflag= 1;
			}
			if ($id3 < 10) {
			    $id33 = "0".$id3;
			} else {
			    $id33= "$id3";
			}
		    }
		    if ($id33 != 0) {
			my $bgdayindx = -1;
			$bgdayindx++ until @bg_day_ids[$bgdayindx]==$id33 or $bgdayindx>$nbg_day_ids;
			$nzbkg = $bg_day_nzs[$bgdayindx];
			$nzrate_uncorr = $baddetrat * $nzrate;
			$dsupweight = $indtgti[$ii-1] * $nzrate_uncorr / $nzbkg;
			#$dsupweight = $indtgti[$ii-1];
			$supweights[$bgdayindx] += $dsupweight;
			#print "$bgdayindx $nzbkg $nzrate_uncorr $dsupweight $supweights[$bgdayindx]\n";
		    } 
		}
	    }
	}
	if ($keep_flag == 1) {
            $nkeep += 1;
            $tkeep += $indtgti[$ii-1];
            $sumibg += $ibgrate*$indtgti[$ii-1];
            $sumhrej += $hrejrate*$indtgti[$ii-1];
	    $sumhbg += $hbgrate*$indtgti[$ii-1];
        } else {
            $nreject += 1;
            $treject += $indtgti[$ii-1];
	    # remove this GTI from event file used to extract the total spectrum
            my $ndelete = $nkeep+1;
	    $cmd = "ftdelrow infile='$tmpgti"."[GTI]' ".
		"outfile='$tmpgti' ".
		"rows='$ndelete' ".
		"confirm='yes' ".
		"clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
	    print "$cmd\n" if ($chatter >= 3);
	    system($cmd); 
	    die "ERROR: failed to delete row from redefined GTI file" if ($?)
	}
	unlink "tmp_clfile_nobaddet_newgti_$ii.gti";
    }
    $exp_bkg += $tkeep;
    if ($nkeep > 0) {
        $ibgav =  $sumibg / $tkeep;
        $hrejav =  $sumhrej / $tkeep;
        $hbgav =  $sumhbg / $tkeep;
        print "Exposure time used: $tkeep, Exposure time discarded: $treject\n";
        print "Average IBG rate in used GTI: $ibgav\n";
        print "Average HREJ rate in used GTI: $hrejav\n";
        print "Average HBG rate in used GTI: $hbgav\n";
	close GTILIST;
        if ($cleanup) { push @tmpfiles, $tmpgtilst };
        ## merge the GTI to be kept
        #$cmd = "ftmgtime ".
          #"ingtis="."'@"."$tmpgtilst' ".
          #"outgti='$tmpgti' ".
          #"merge='OR' ".
          #"clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
	#print "$cmd\n" if ($chatter >= 3);
	#system($cmd);    # Create a new GTI file from the GTI extension of an existing event file, bounding such that dtmin<STOP-START<dtmax
	#die "ERROR: ftmgtime failed to create GTI for total spectrum" if ($?);
	#die "ERROR:  Total spectrum GTI not produced" if (! -f $tmpgti);
	#$subgtierror = SubstituteGTI("tmp_clfile_nobaddet_newgti.evt","STDGTI",$tmpgti,$histstr,$chatter);
	$subgtierror = SubstituteGTI("tmp_clfile_nobaddet_newgti.evt","GTI",$tmpgti,$histstr,$chatter);
        if ($cleanup) { push @tmpfiles, $tmpgti };
        # Extract the total spectrum
        $cmd = "extractor ".
          "filename='tmp_clfile_nobaddet_newgti.evt' ".
          "eventsout='NONE' ". "imgfile='NONE' ".
          "phafile='$totspec' ".
          "wtmapb='no' ". "fitsbinlc='NONE' ". "binlc='16.0' ".
          "lcthresh='0.0' ". "regionfile='NONE' ". "timefile='NONE' ". 
          "xcolf='RAWX' ". "ycolf='RAWY' ". "tcol='TIME' ". "ecol='PI' ".
          "xcolh='RAWX' ". "ycolh='RAWY' ". "phamax='TLMAX' ". 
          "xfkey='TLMAX' ". "yfkey='TLMAX' ". 
          "xhkey='TLMAX' ". "yhkey='TLMAX' ". "clobber='$clobstr' ";
	print "$cmd\n" if ($chatter >= 3);
	system($cmd);
	die "ERROR: extractor failed to create total spectrum" if ($?);
	die "ERROR: new total spectrum not produced" if (! -f $totspec);
        my @weights = map {$_ / $exp_bkg} @weights;
        my @exp_weights = map {$_ / $exp_bkg} @exp_weights;
        my @supweights = map {$_ / $exp_bkg} @supweights;
	#print "$exp_bkg\n";
	#for (my $ii=0; $ii<$nbg_ngt_ids; $ii+=1) {
	    #print "$bg_ngt_ids[$ii], $bg_ngt_ibgs[$ii], $bg_ngt_names[$ii], $weights[$ii], $exp_weights[$ii]\n";
	#}
	#for (my $ii=0; $ii<$nbg_day_ids; $ii+=1) {
	    #print "$bg_day_ids[$ii], $bg_day_nzs[$ii], $bg_day_names[$ii], $supweights[$ii]\n";
	#}
	#$wterror = CreateWeightedBkgSpec(\@bg_ngt_expos,\@weights,\@exp_weights,\@supweights,$bkgspec,$chatter,$clobstr);
        # Move copies of library spectra to local directory --
        # mathpha doesn't like full path, and doesn't like long strings
        for (my $ll=0; $ll<$nbg_ngt_ids; $ll+=1) {
	    if ($weights[$ll] > 0.0) {
                $namein = "$bkgdir" . "/" . "$bg_ngt_names[$ll]";
                $nameout = "tmp_ngtbg" ."$ll" .".pha";
		$cmd = "ftcopy ". "infile='$namein' ". "outfile='$nameout' ". "clobber=yes";
		print "$cmd\n" if ($chatter >= 3);
		system($cmd);
                die "ERROR: ftcopy failed to copy nighttime library spectrum" if ($?);
                die "ERROR: nighttime library spectrum $namein not copied" if (! -f $nameout );
                if ($cleanup) { push @tmpfiles, $nameout };
	    }
	}
        for (my $ll=0; $ll<$nbg_day_ids; $ll+=1) {
	    if ($supweights[$ll] > 0.0) {
                $namein = "$bkgdir" . "/" . "$bg_day_names[$ll]";
                $nameout = "tmp_daybg" ."$ll" .".pha";
		$cmd = "ftcopy ". "infile='$namein' ". "outfile='$nameout' ". "clobber=yes";
		print "$cmd\n" if ($chatter >= 3);
		system($cmd);
                die "ERROR: ftcopy failed to copy daytime library spectrum" if ($?);
                die "ERROR: nighttime library spectrum $namein not copied" if (! -f $nameout );
                if ($cleanup) { push @tmpfiles, $nameout };
	    }
	}
        # Make the weighted background spectrum
	$wterror = CreateWeightedBkgSpec(\@bg_ngt_expos,\@weights,\@exp_weights,\@supweights,\@bg_ngt_names,\@bg_day_names,\$bkgspec,\$chatter,\$clobstr);
	die "ERROR: background spectrum not produced on return from CreateWeightedBkgSpec" if (! -f $bkgspec)
    }
    #if ($cleanup) { 
        #@tmpfiles = glob("tmp_*");
    #} 
    return $status;
}

sub parse_fpm_list ($) {
  my ($fpmstr) = (@_);
  my (@fpmlist);

  my @fpmvals = split(/,/,"$fpmstr");
  foreach my $fpmval (@fpmvals) {
    if ($fpmval =~ m/^\d+$/) {
      push @fpmlist, $fpmval;
    } elsif ($fpmval =~ m/^(\d+)-(\d+)$/) {
      push @fpmlist, ($1 .. $2);
    } else {
      die "ERROR: FPM slice number specification $fpmval not recognized";
    }
  }

  return @fpmlist;
}

sub createStartStopGTI ($$$$$) {

    # Create a new GTI file from the GTI extension of an existing event file, bounding such that dtmin<STOP-START<dtmax
    my $clfile = shift;
    my $dtmin = shift;
    my $dtmax = shift;
    my $histstr = shift;
    my $chatter = shift;

    my $status = 0;
    my $hdrfile = "tmp_hdrfile";
    my $cdfile = "tmp_cdfile";
    my $datafile = "tmp_datafile";

    $ingti = SimpleFITS->open("$clfile", type=>"table", access=>"read");
    die "ERROR: Failed to open cleaned event file" if (! $ingti );
    $ingti->move("GTI");
    $ingti->readkey("NAXIS2",$inngti);
    $ingti->readkey("MJDREFI",$mjdrefi);
    $ingti->readkey("MJDREFF",$mjdreff);
    $ingti->readkey("TIMEZERO",$timezero);
    if (length($timezero) == 0) { $ingti->setstatus(0) };
    @ingtistart = $ingti->readcol("START",{type=>TDOUBLE});
    @ingtistop = $ingti->readcol("STOP",{type=>TDOUBLE});
    @indtgti = $ingti->calculate("STOP-START",{type=>TDOUBLE});
    $ingti->close();

    open HDRFILE, ">", $hdrfile;
    print HDRFILE "EXTNAME 'GTI'\n";
    print HDRFILE "HDUCLAS1 'GTI' / Contains good time intervals\n";
    print HDRFILE "MJDREFI '" .$mjdrefi. "'" ."\n";
    print HDRFILE "MJDREFF '" .$mjdreff. "'" ."\n" ;
    if (length($timezero) != 0) {print HDRFILE "TIMEZERO '$timezero' / Zero-point offset for TIME column\n" };
    close HDRFILE;
    die "ERROR: GTI header file not produced in createStartStopGTI" if (! -f $hdrfile);
    if ($cleanup) { push @tmpfiles, $hdrfile };

    open CDFILE, ">", $cdfile;
    print CDFILE "START 1D s\n";
    print CDFILE "STOP 1D s\n";
    close CDFILE;
    die "ERROR: GTI coldef file not produced in createStartStopGTI" if (! -f $cdfile);
    if ($cleanup) { push @tmpfiles, $cdfile };

    open DATAFILE, ">", $datafile;

    for (my $ii=0; $ii<$inngti; $ii+=1) {
	if ($indtgti[$ii] > $dtmax) {
	    my $nextstart = $ingtistart[$ii];
            my $nn = int($indtgti[$ii]/$dtmax);
	    for (my $jj=0; $jj<$nn; $jj+=1) {
		my $newstart = $nextstart;
                my $newstop = $nextstart + $dtmax;
                $nextstart = $newstop;
		print DATAFILE "$newstart $newstop\n";
	    }
	    if (fmod($indtgti[$ii],$dtmax) > $dtmin) {
		$newstart = $nextstart;
                $newstop = $ingtistop[$ii];
		print DATAFILE "$newstart $newstop\n";
	    }

	} elsif ($indtgti[$ii] > $dtmin) {
	    print DATAFILE "$ingtistart[$ii] $ingtistop[$ii]\n";
	}
    }

    close DATAFILE;
    die "ERROR: GTI data file not produced in createStartStopGTI" if (! -f $datafile);
    if ($cleanup) { push @tmpfiles, $datafile };

    $cmd = "ftcreate ".
      "cdfile='tmp_cdfile' ".
      "datafile='tmp_datafile' ".
      "outfile='tmp_newgti.gti' ".
      "headfile='tmp_hdrfile' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: ftcreate failed to make new gti file in createStartStopGTI" if ($?);
    die "ERROR: new GTI file not produced in createStartStopGTI" if (! -f "tmp_newgti.gti");
    if ($cleanup) { push @tmpfiles, "tmp_newgti.gti" };

    return $status;
}

sub SubstituteGTI ($$$$$) {

    # Subsititue and apply a new GTI to an event file
    my $infile = shift;
    my $ext_to_add = shift;
    my $gti_to_add = shift;
    my $histstr = shift;
    my $chatter = shift;

    my $status = 0;
    my $ext_to_del = "GTI";

    $cmd = "ftdelhdu infile='$infile"."[GTI]' ".
      "outfile='NONE' ".
      "confirm='yes' ".
      "clobber='yes' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: failed to delete GTI hdu in SubstituteGTI" if ($?);

    $cmd = "ftappend infile='$gti_to_add"."[$ext_to_add]' ".
      "outfile='$infile' ".
      "history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: failed to append new hdu in SubstituteGTI" if ($?);

    $cmd = "ftcopy infile='$infile"."[EVENTS][gtifilter()]' ".
      "outfile='$infile' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: failed to filter on new GTI in SubstituteGTI" if ($?);

    return $status;
}

sub RateGTI ($$$$$) {

    # Calculate and return an event rate from an input file filtered with an inpout GTI
    # The delta-time is pre-calculated - could use gtisum locally
    # A scale-factor (here to account for <52 MPUs) is applied
    my $infile = shift;
    my $gtifile = shift;
    my $dt = shift;
    my $scale = shift;
    my $histstr = shift;
    my $chatter = shift;

    #my $status = 0;

    my $gtiexpr = "gtifilter(\"$gtifile\")";
    $cmd = "ftcopy infile='$infile"."[EVENTS][$gtiexpr]' ".
      "outfile='tmp_single_gti.evt' ".
      "copyall='yes' ".
      "clobber='yes' "."history='$histstr' "."chatter='$chatter' ";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: failed to filter on single GTI in RateGTI" if ($?);
    die "ERROR: new single gti event file not produced in RateGTI" if (! -f "tmp_single_gti.evt");
    if ($cleanup) { push @tmpfiles, "tmp_single_gti.evt" };

    $tmp_single_gti_evt = SimpleFITS->open("tmp_single_gti.evt", type=>"table", access=>"read");
    $tmp_single_gti_evt->move("EVENTS");
    $tmp_single_gti_evt->readkey("NAXIS2",$num_single_gti_evt);
    $tmp_single_gti_evt->close();

    my $gtirate = $num_single_gti_evt / $scale / $dt;
    #print "$num_single_gti_evt $dt $gtirate\n";

    #return $status;
    return $gtirate;
}

sub FindIBGID ($$) {

    # Get the 3C50_RGv5 hrej ID from ibg ID and hrej value
    my $ibg_id = shift;
    my $hrej = shift;

    my $hreg_id = 0;

    if ($ibg_id == 1) {
	if ($hrej < 0.15) {
	    $hrej_id = 1;
	} elsif ($hrej < 0.40) {
	    $hrej_id = 2;
	} elsif ($hrej < 1.0) {
	    $hrej_id = 3;
	} else {
	    $hrej_id = 4;
	}	
    } elsif ($ibg_id == 2) {
	if ($hrej < 0.20) {
	    $hrej_id = 1;
	} elsif ($hrej < 0.40) {
	    $hrej_id = 2;
	} elsif ($hrej < 1.0) {
	    $hrej_id = 3;
	} else {
	    $hrej_id = 4;
	}
    } elsif ($ibg_id == 3) {
	if ($hrej < 0.25) {
	    $hrej_id = 1;
	} elsif ($hrej < 0.40) {
	    $hrej_id = 2;
	} elsif ($hrej < 1.0) {
	    $hrej_id = 3;
	} else {
	    $hrej_id = 4;
	}
    } elsif ($ibg_id == 4) {
	if ($hrej < 0.13) {
	    $hrej_id = 1;
	} elsif ($hrej < 0.40) {
	    $hrej_id = 2;
	} elsif ($hrej < 0.90) {
	    $hrej_id = 3;
	} else {
	    $hrej_id = 4;
	}
    } elsif ($ibg_id == 5) {
	if ($hrej < 0.20) {
	    $hrej_id = 1;
	} elsif ($hrej < 0.40) {
	    $hrej_id = 2;
	} elsif ($hrej < 0.90) {
	    $hrej_id = 3;
	} else {
	    $hrej_id = 4;
	}
    } elsif ($ibg_id == 6) {
	if ($hrej < 0.55) {
	    $hrej_id = 2;
	} elsif ($hrej < 0.85) {
	    $hrej_id = 3;
	} elsif ($hrej < 2.0) {
	    $hrej_id = 4;
	} else {
	    $hrej_id = 5;
	}
    } else {
	$hrej_id = 0;
    }
    #print "$ibg_id $hrej $hrej_id\n";
    return $hrej_id;
}

sub CreateWeightedBkgSpec {

    # Use mathpha to create a weighted spectrum with weighted exposure
    # Weights and library spectrum names are input
    my ($expos_ref, $wts_ref, $expwts_ref, $supwts_ref, $ngtbg_names_ref, $daybg_names_ref, $bkgspec_ref, $chat_ref, $clob_ref) = @_;
    my @expos = @{ $expos_ref }; 
    my @wts = @{ $wts_ref };
    my @expwts = @{ $expwts_ref };
    my @supwts = @{ $supwts_ref };
    my @ngtbg_names = @{ $ngtbg_names_ref };
    my @daybg_names = @{ $daybg_names_ref };
    my $bkgspec = $ { $bkgspec_ref };
    my $chatter = $ { $chat_ref };
    my $clobber = $ { $clob_ref };

    #print "$bkgspec\n";
    my $bgspec_expo = 0.0;
    for (my $ii=0; $ii<scalar @expos; $ii+=1) {
	$bgspec_expo += $expwts[$ii] * $expos[$ii];
	#print "$wts[$ii] $expwts[$ii] $expos[$ii] $bgspec_expo\n";
    }

    # Provide for limited number of input spectrum into mathpha...
    # ... by (possibly) running it twice with maximum of 16 entries 
    my $mathpha_expr1 = "";
    my $mathpha_expr2 = "";
    my $first = 1;
    my $numspec = 0;
    #print "$mathpha_expr1\n";
    for (my $ii=0; $ii<scalar @wts; $ii+=1) {
	$wt_str = mathpha_str($wts[$ii]);
	if ($wts[$ii] > 0.0) {
	    $numspec+=1;
	    if ($first==1) {
		if ($numspec<15) {
		    $expr_to_add1 = $wt_str."*tmp_ngtbg" ."$ii" .".pha";
		    $mathpha_expr1 = $mathpha_expr1.$expr_to_add1;
		} else {
		    $expr_to_add2 = $wt_str."*tmp_ngtbg" ."$ii" .".pha";
		    $mathpha_expr2 = $mathpha_expr2.$expr_to_add2;
		}
		$first = 0;
	    } else {
		if ($numspec<15) {
		    $expr_to_add1 = "+".$wt_str."*tmp_ngtbg" ."$ii" .".pha";
		    $mathpha_expr1 = $mathpha_expr1.$expr_to_add1;
		    if ($numspec==14) {$first=1};
		} else {
		    $expr_to_add2 = "+".$wt_str."*tmp_ngtbg" ."$ii" .".pha";
		    $mathpha_expr2 = $mathpha_expr2.$expr_to_add2;
		}
	    }
	}
    }

    for (my $ii=0; $ii<scalar @supwts; $ii+=1) {
	#print "$supwts[$ii]\n";
	$swt_str = mathpha_str($supwts[$ii]);
	if ($supwts[$ii] > 0.0) {
	    $numspec+=1;
	    if ($first==1) {
		if ($numspec<15) {
		    $expr_to_add1 = $swt_str."*tmp_daybg" ."$ii" .".pha";
		    $mathpha_expr1 = $mathpha_expr1.$expr_to_add1;
		} else {
		    $expr_to_add2 = $swt_str."*tmp_daybg" ."$ii" .".pha";
		    $mathpha_expr2 = $mathpha_expr2.$expr_to_add2;
		}
		$first = 0;
	    } else {
		if ($numspec<15) {
		    $expr_to_add1 = "+".$swt_str."*tmp_daybg" ."$ii" .".pha";
		    $mathpha_expr1 = $mathpha_expr1.$expr_to_add1;
		    if ($numspec==14) {$first=1};
		} else {
		    $expr_to_add2 = "+".$swt_str."*tmp_daybg" ."$ii" .".pha";
		    $mathpha_expr2 = $mathpha_expr2.$expr_to_add2;
		}
	    }
	}
    }	

    print "$mathpha_expr1\n" if ($chatter >= 3);
    $cmd = "mathpha ".
      "expr='$mathpha_expr1' ". "units='R' ".
      "outfil='tmp_bkg.pi' ". "exposure='$bgspec_expo' ".
      "areascal='%' ". "properr='yes' ". "errmeth='gaussian' ".
      "ncomments='0' ". "clobber=yes";
    print "$cmd\n" if ($chatter >= 3);
    system($cmd);
    die "ERROR: mathpha failed to create temp bkg spectrum" if ($?);
    die "ERROR:  Temp bkg spectrum not produced" if (! -f "tmp_bkg.pi");
    if ($cleanup) { push @tmpfiles, "tmp_bkg.pi" };
    if ($numspec<15){
	$cmd = "ftcopy ".
	  "infile='tmp_bkg.pi' ".
	  "outfile='$bkgspec' ".
	  "copyall='yes' ".
	  "clobber='$clobber' "."history='$histstr' "."chatter='$chatter' ";
	print "$cmd\n" if ($chatter >= 3);
	system($cmd);    
        die "ERROR: ftcopy failed to copy temp to bkg spectrum" if ($?);
    } else {
        $outexpr = $mathpha_expr2 . "+1.0*" . "tmp_bkg.pi";
	$cmd = "mathpha ".
	  "expr='$outexpr' ". "units='R' ".
	  "outfil='$bkgspec' ". "exposure='$bgspec_expo' ".
	  "areascal='%' ". "properr='yes' ". "errmeth='gaussian' ".
	  "ncomments='0' ". "clobber=$clobber";
	print "$cmd\n" if ($chatter >= 3);
	system($cmd);    
        die "ERROR: mathpha failed to final bkg spectrum" if ($?);
    }

    #my $test = 3.43e-7;
    #$testout = mathpha_str($test);
    die "ERROR: background spectrum not produced in CreateWeightedBkgSpec" if (! -f $bkgspec);
    return $status;
}

sub mathpha_str($) {

    # Convert weights from exponential to fractional notation for use in mathpha
    my $mstr = shift;
    my $idx = index($mstr, "e");
    #print "$mstr $idx\n";
    if ($idx != -1) {
	#$nzeros = substr $mstr, -2;
	#$nzeros = int(substr $mstr, -2)-1;
	$nzeros = (substr $mstr, -2)-1;
        $nnum1 = substr $mstr, 0, 1;
        $nnum2 = substr $mstr, 2, $idx-2;
	my $zeros = "0.";
	for (my $ii=0; $ii<$nzeros; $ii+=1) {$zeros=$zeros."0"}
	$newnum = $zeros.$nnum1.$nnum2;
	#print "$newnum\n";
	return $newnum;
    } else {
	return $mstr;
    }

    return $newnum;
}
